<?xml version="1.0" encoding="ISO-8859-1"?>

<!-- 
This XSL stylesheet is for converting the XML data of the jClimber's Diary
application to HTML. After XSL transformation the all the "${$TAG_NAME$}$"
needs to be replaced with the appropriate resuurce strings. All the
"$($TAG_NAME$)$" tags need to be replaced with the command output (e.g.
the export date).
(C) by Stefan Saring, 2002/03/24
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

    <!-- Output for the whole page, which contains all subitems. -->
    <xsl:template match="climbersdiary">
        <html>
            <title> ${$export.title.html$}$ </title>
            <body bgcolor="#FFFFFF">
            
                <center><h1> ${$export.title.html$}$ </h1></center>
                
                <!-- Print out the total number of areas, groups,... in a table. -->
                <!-- Disabled for export, but works very good :-) 
                <hr/>
                <h2>Statistik:</h2>
                <blockquote>
                    <table>
                        <tr>
                            <td>Anzahl Klettergebiete:</td>
                            <td><xsl:value-of select="count(//area)"/></td>
                        </tr>
                        <tr>
                            <td>Anzahl Felsgruppen:</td>
                            <td><xsl:value-of select="count(//group)"/></td>
                        </tr>
                        <tr>
                            <td>Anzahl Klettergipfel:</td>
                            <td><xsl:value-of select="count(//rock)"/></td>
                        </tr>
                        <tr>
                            <td>Anzahl Kletterrouten:</td>
                            <td><xsl:value-of select="count(//route)"/></td>
                        </tr>
                    </table>
                </blockquote> -->

                <!-- Scan for area subitems. -->
                <xsl:apply-templates select="area"/>
                
                <!-- append generator comments at bottom -->
                <hr/>
                <p><center><i> ${$export.generator.comment$}$ $($export.date$)$ </i></center></p>
                
            </body>
        </html>
    </xsl:template>


    <!-- Output for a climbing area. -->
    <xsl:template match="//area">
        <hr/>
        <h2> ${$export.area.label$}$ <xsl:value-of select="name"/> </h2>

        <!-- Print comment only if available. -->
        <xsl:if test="comment != ''">
            <xsl:value-of select="comment"/>
        </xsl:if>

        <!-- Scan for group or rock subitems. -->
        <xsl:apply-templates select="group"/>
        <xsl:apply-templates select="rock"/>
    </xsl:template>


    <!-- Output for a climbing group. -->
    <xsl:template match="//group">
        <blockquote>
            <h3> ${$export.group.label$}$ <xsl:value-of select="name"/> </h3>

            <!-- Print comment only if available. -->
            <xsl:if test="comment != ''">
                <xsl:value-of select="comment"/>
            </xsl:if>

            <!-- Scan for group or rock subitems. -->
            <xsl:apply-templates select="group"/>
            <xsl:apply-templates select="rock"/>
        </blockquote>
    </xsl:template>


    <!-- Output for a climbing rock. -->
    <xsl:template match="//rock">
        <blockquote>
            <h3> ${$export.rock.label$}$ <xsl:value-of select="name"/> </h3>

            <!-- Print comment only if available. -->
            <xsl:if test="comment != ''">
                <xsl:value-of select="comment"/>
            </xsl:if>
            <!-- Scan for route subitems. -->
            <xsl:apply-templates select="route"/>
        </blockquote>
    </xsl:template>


    <!-- Output for a climbing route (has no childs). -->
    <xsl:template match="//route">
        <p>
            <table width="100%">
                <tr bgcolor="#90B6D9">
                    <td width="1%"><b> ${$export.route.label$}$ </b></td>
                    <td><b>
                        <xsl:value-of select="name"/>
                        <!-- i don't know how to make an OR statement :-) -->
                        <xsl:if test="@completed = '0'"> ${$export.route.notcompleted$}$ </xsl:if>
                        <xsl:if test="@completed = 'false'"> ${$export.route.notcompleted$}$ </xsl:if>
                    </b></td>                    
                </tr>

                <tr>
                    <td> ${$export.route.difficulty$}$ </td>
                    <td><xsl:value-of select="difficulty"/></td>
                </tr>

                <tr>
                    <td> ${$export.route.lead$}$ </td>
                    <td>
                        <!-- Test lead flag (0 or 1) and do text output. -->
                        <xsl:choose>
                            <xsl:when test="lead[. = 0]"> ${$export.route.lead.no$}$ </xsl:when>
                            <xsl:when test="lead[. = 1]"> ${$export.route.lead.yes$}$ </xsl:when>
                            <xsl:otherwise> </xsl:otherwise>
                        </xsl:choose>
                    </td>
                </tr>

                <tr>
                    <td> ${$export.route.date$}$ </td>
                    <td>
                        <!-- Extract day, month and your of the date string and store it in variables. -->
                        <xsl:variable name="day">
                            <xsl:value-of select="substring(date,1,string-length(date)-6)"/>
                        </xsl:variable>
                        <xsl:variable name="month">
                            <xsl:value-of select="substring(date,string-length(date)-5,2)"/>
                        </xsl:variable>
                        <xsl:variable name="year">
                            <xsl:value-of select="substring(date,string-length(date)-3,4)"/>
                        </xsl:variable>
                        <!-- Formated output of the day. -->
                        <xsl:value-of select="$day"/>.<xsl:value-of select="$month"/>.<xsl:value-of select="$year"/>
                    </td>
                </tr>

                <!-- Print comment only if available. -->
                <xsl:if test="comment != ''">
                    <tr valign="top">
                        <td> ${$export.route.comment$}$ </td>
                        <td><xsl:value-of select="comment"/></td>
                    </tr>
                </xsl:if>
            </table>
        </p>
    </xsl:template>

</xsl:stylesheet>

