package de.saring.afw;

import java.util.Locale;
import java.util.ResourceBundle;
import java.util.MissingResourceException;

/**
 * Generic class for reading resources from java-properties-files for an specified 
 * locale. It uses the ResourceBundle class and adds some error-evaluation code.
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class AFWResourceReader extends Object
{
    /**
     * This ResourceBundle contains all the application resources.
     */
    private ResourceBundle resBundle;

    /**
     * Constructs an instance of this class.
     */
    public AFWResourceReader ()
    {
        super ();
    }

    /**
     * Reads a properties file for the specified locale into the intern
     * ResourceBundle object. Take a look at java.util.ResourceBundle
     * for correct file and locale names.
     *
     * @param filename The base filename of the resource file.
     * @param locale The locale for the resource file.
     * @return True if the resources were read successfully.
     */
    public boolean readResourceBundle (String filename, Locale locale)
    {
        try {
            this.resBundle = ResourceBundle.getBundle (filename, locale);
        }
        catch (MissingResourceException e) {
            System.err.println ("Error: AFWResourceReader: " + e);
            return false;
        }
        return true;
    }

    /**
     * Returns the value of the string resource for the specified key.
     * 
     * @param resourceKey The key (name) of the resource.
     * @return The resource string or "???", if the resource is not available.
     */
    public String getResourceString (String resourceKey)
    {
        if (resBundle == null)
            return new String ("???");
        
        try {
            return resBundle.getString (resourceKey);
        }
        catch (MissingResourceException e) {
            System.err.println ("Error: AFWResourceReader: Key '" +
                resourceKey + "' not found ...");
            return new String ("???");
        }
    }
}
