package de.saring.jcdiary;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.net.URL;
import de.saring.afw.AFWView;
import de.saring.jcdiary.tree.*;
import de.saring.jcdiary.dialogs.*;
import de.saring.jcdiary.misc.HelpViewer;

/**
 * JCDView is the only view-class of the application. It creates an initializes the
 * complete GUI (menus, tree, ...). Many of the user-interactions (e.g. messageboxes
 * before moving items) are implemented in the methods of this class.
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class JCDView extends AFWView
{
    /** all the GUI controls of the view */
    private JMenuItem   miExport, miQuit, miHelp, miAbout;
    private JMenuItem   miAddArea, miAddGroup, miAddRock, miAddRoute;
    private JMenuItem   miProperties, miCountRoutes, miDelete, miSearch;
    private JMenu       menuAdd, menuEntry;
    private JPopupMenu  pmEntry;
    private JMenuItem   pmiAdd, pmiAddArea, pmiAddGroup, pmiAddRock, pmiAddRoute;
    private JMenuItem   pmiProperties, pmiCountRoutes, pmiDelete;
    private JButton     tbExport, tbProperties, tbCountRoutes, tbDelete;
    private JButton     tbFind, tbHelp, tbAbout;
    public  JTree       tree;

    public JCDView ()
    {
        super ();
        initGUI ();
        // setLocale (Locale.GERMAN);

        // set frame icon and title
        URL urlLogo = this.getClass ().getResource ("/resource/images/jcdiary.gif");
        JCDApp.getFrame ().setIconImage (getToolkit().getImage (urlLogo));
        String appTitle = JCDApp.getResReader ().getResourceString ("app.name") +
            " " + JCDApp.getResReader ().getResourceString ("app.version");
        JCDApp.getFrame ().setTitle (appTitle);

        JCDApp.getFrame ().addWindowListener (new WindowAdapter () {
            public void windowClosing(WindowEvent event) {
                quitApplication ();
            }
        });
    }

    /**
     * Initializes the GUI of the application.
     */
    private void initGUI ()
    {
        // Menu-design
        JMenuBar mbar = new JMenuBar ();

        // File-menu
        // --> Export to HTML
        JMenu menu1 = new JMenu (JCDApp.getResReader ().getResourceString ("menu.file.label"));
        miExport = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.file.export.label"));
        miExport.addActionListener (new ExportDiaryListener ());
        menu1.add (miExport);
        // --> Quit application
        miQuit = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.file.exit.label"));
        miQuit.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                quitApplication ();
            }
        });
        menu1.add (miQuit);
        mbar.add (menu1);

        
        // Add entry-menu
        menuAdd = new JMenu (JCDApp.getResReader ().getResourceString ("menu.add.label"));
        // -> add climbing area
        miAddArea = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.area.label"));
        miAddArea.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addArea ();
            }
        });
        miAddArea.setEnabled (true);
        menuAdd.add (miAddArea);
        // -> add rock group
        miAddGroup = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.group.label"));
        miAddGroup.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addGroup ();
            }
        });
        miAddGroup.setEnabled (false);
        menuAdd.add (miAddGroup);
        // -> add climbing rock
        miAddRock = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.rock.label"));
        miAddRock.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addRock ();
            }
        });
        miAddRock.setEnabled (false);
        menuAdd.add (miAddRock);
        // -> add climbing route
        miAddRoute = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.route.label"));
        miAddRoute.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addRoute ();
            }
        });
        miAddRoute.setEnabled (false);
        menuAdd.add (miAddRoute);
        mbar.add (menuAdd);

        
        // Entry-menu
        menuEntry = new JMenu (JCDApp.getResReader ().getResourceString ("menu.entry.label"));
        // -> entry properties
        miProperties = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.properties.label"));
        miProperties.addActionListener (new EntryEditListener ());
        miProperties.setEnabled (false);
        menuEntry.add (miProperties);
        // -> tally climbing routes
        miCountRoutes = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.count.label"));
        miCountRoutes.addActionListener (new CountRoutesListener ());
        miCountRoutes.setEnabled (false);
        menuEntry.add (miCountRoutes);
        // --> delete entry
        miDelete = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.delete.label"));
        miDelete.addActionListener (new EntryDeleteListener ());
        miDelete.setEnabled (false);
        menuEntry.add (miDelete);
        menuEntry.addSeparator ();
        // --> search entry
        miSearch = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.search.label"));
        miSearch.addActionListener (new EntryFindListener ());
        menuEntry.add (miSearch);
        mbar.add (menuEntry);

        
        // Help-menu
        JMenu menuHelp = new JMenu (JCDApp.getResReader ().getResourceString ("menu.help.label"));
        // --> open online help window
        miHelp = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.help.content.label"));
        miHelp.addActionListener (new OnlineHelpListener ());
        menuHelp.add (miHelp);
        // -> show about box
        miAbout = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.help.about.label"));
        miAbout.addActionListener (new AppAboutListener ());
        menuHelp.add (miAbout);
        mbar.add (menuHelp);

        // set the menubar into the frame
        JCDApp.getFrame ().setJMenuBar (mbar);

        
        // create the popup-menu
        pmEntry = new JPopupMenu (JCDApp.getResReader ().getResourceString ("menu.entry.label"));
        pmiProperties = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.properties.label"));
        pmiProperties.addActionListener (new EntryEditListener ());
        pmEntry.add (pmiProperties);
        pmEntry.addSeparator ();
        pmiAdd = new JMenu (JCDApp.getResReader ().getResourceString ("menu.add.label"));
        pmEntry.add (pmiAdd);
        pmiAddArea = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.area.label"));
        pmiAddArea.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addArea ();
            }
        });
        pmiAdd.add (pmiAddArea);
        pmiAddGroup = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.group.label"));
        pmiAddGroup.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addGroup ();
            }
        });
        pmiAdd.add (pmiAddGroup);
        pmiAddRock = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.rock.label"));
        pmiAddRock.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addRock ();
            }
        });
        pmiAdd.add (pmiAddRock);
        pmiAddRoute = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.add.route.label"));
        pmiAddRoute.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                addRoute ();
            }
        });
        
        pmiAdd.add (pmiAddRoute);                
        pmEntry.addSeparator ();
        pmiCountRoutes = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.count.label"));
        pmiCountRoutes.addActionListener (new CountRoutesListener ());
        pmEntry.add (pmiCountRoutes);
        pmiDelete = new JMenuItem (JCDApp.getResReader ().getResourceString ("menu.entry.delete.label"));
        pmiDelete.addActionListener (new EntryDeleteListener ());
        pmEntry.add (pmiDelete);

        
        // create the toolbar and entries
        JToolBar toolbar = new JToolBar ();
        toolbar.setRollover (true);
        toolbar.setBorderPainted (false);
        // -> export diary - button
        URL urlImgExport = this.getClass ().getResource ("/resource/images/toolbar/export.png");
        tbExport = new JButton (new ImageIcon (getToolkit().getImage (urlImgExport)));
        tbExport.setRequestFocusEnabled (false);
        tbExport.addActionListener (new ExportDiaryListener ());
        tbExport.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.export.tooltip"));
        toolbar.add (tbExport);
        // -> find entry - button
        URL urlImgFind = this.getClass ().getResource ("/resource/images/toolbar/find.png");
        tbFind = new JButton (new ImageIcon (getToolkit().getImage (urlImgFind)));
        tbFind.setRequestFocusEnabled (false);
        tbFind.addActionListener (new EntryFindListener ());
        tbFind.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.find.tooltip"));
        toolbar.add (tbFind);
        toolbar.addSeparator ();
        // -> edit entry - button
        URL urlImgEdit = this.getClass ().getResource ("/resource/images/toolbar/edit.png");
        tbProperties = new JButton (new ImageIcon (getToolkit().getImage (urlImgEdit)));
        tbProperties.setRequestFocusEnabled (false);
        tbProperties.addActionListener (new EntryEditListener ());
        tbProperties.setEnabled (false);
        tbProperties.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.properties.tooltip"));
        toolbar.add (tbProperties);
        // -> count routes - button
        URL urlImgCount = this.getClass ().getResource ("/resource/images/toolbar/count.png");
        tbCountRoutes = new JButton (new ImageIcon (getToolkit().getImage (urlImgCount)));
        tbCountRoutes.setRequestFocusEnabled (false);
        tbCountRoutes.addActionListener (new CountRoutesListener ());
        tbCountRoutes.setEnabled (false);
        tbCountRoutes.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.count.tooltip"));
        toolbar.add (tbCountRoutes);
        // -> delete entry - button
        URL urlImgDelete = this.getClass ().getResource ("/resource/images/toolbar/delete.png");
        tbDelete = new JButton (new ImageIcon (getToolkit().getImage (urlImgDelete)));
        tbDelete.setRequestFocusEnabled (false);
        tbDelete.addActionListener (new EntryDeleteListener ());
        tbDelete.setEnabled (false);
        tbDelete.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.delete.tooltip"));
        toolbar.add (tbDelete);
        toolbar.addSeparator ();
        // -> online help - button
        URL urlImgHelp = this.getClass ().getResource ("/resource/images/toolbar/help.png");
        tbHelp = new JButton (new ImageIcon (getToolkit().getImage (urlImgHelp)));
        tbHelp.setRequestFocusEnabled (false);
        tbHelp.addActionListener (new OnlineHelpListener ());
        tbHelp.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.help.tooltip"));
        toolbar.add (tbHelp);
        // -> show about box - button
        URL urlImgAbout = this.getClass ().getResource ("/resource/images/toolbar/about.png");
        tbAbout = new JButton (new ImageIcon (getToolkit().getImage (urlImgAbout)));
        tbAbout.setRequestFocusEnabled (false);
        tbAbout.addActionListener (new AppAboutListener ());
        tbAbout.setToolTipText (JCDApp.getResReader ().getResourceString ("toolbar.about.tooltip"));
        toolbar.add (tbAbout);
        // add toolbar to panel
        this.setLayout (new BorderLayout ());
        add (toolbar, BorderLayout.NORTH);
        
        
        // create the diary tree
        tree = new JTree (((JCDDocument) JCDApp.getDocument ()).treeModel);
        tree.setCellRenderer (new JCDCellRenderer ());
        tree.addMouseListener (JCDApp.getMouseControl ());
        tree.addMouseMotionListener (JCDApp.getMouseControl ());

        // only single selections are possible
        DefaultTreeSelectionModel selModel = new DefaultTreeSelectionModel ();
        selModel.setSelectionMode (DefaultTreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.setSelectionModel (selModel);

        // update menu items, when selection in tree has changed
        tree.addTreeSelectionListener (new TreeSelectionListener () {
            public void valueChanged (TreeSelectionEvent e) {
                setMenuItemsStatus ();
            }
        });
        
        // set tree into frame (is scrollable)
        JScrollPane scrollPane = new JScrollPane (tree);
        add (scrollPane, BorderLayout.CENTER);
    }
    
    /**
     * Returns the currently selected node in the tree.
     * @return the selected node or null
     */
    public JCDTreeNode getSelectedNode ()
    {
        TreePath selPath = tree.getSelectionPath ();        

        if (selPath != null)
            return (JCDTreeNode) selPath.getLastPathComponent ();
            
        return null;
    }

    /**
     * This method lets the user to add a new climbing area to the diary (displays
     * the appropriate dialog).
     */
    private void addArea ()
    {
        DiaryItem diTemp = new DiaryItem (DiaryItem.TYPE_AREA, "", "", false, -1, true, "");
        GroupDialog dialog = new GroupDialog (diTemp);
        dialog.setVisible (true);

        if (dialog.isDialogCanceled () == false) {
            ((JCDDocument) JCDApp.getDocument ()).insertArea (diTemp);
        }
    }

    /**
     * This method lets the user to add a new rock group to the diary (displays
     * the appropriate dialog).
     */
    private void addGroup ()
    {
        JCDTreeNode selNode = getSelectedNode ();

        DiaryItem diTemp = new DiaryItem (DiaryItem.TYPE_GROUP, "", "", false, -1, true, "");
        GroupDialog dialog = new GroupDialog (diTemp);
        dialog.setVisible (true);

        if (dialog.isDialogCanceled () == false)
            ((JCDDocument) JCDApp.getDocument ()).insertGroup (diTemp, selNode);
    }

    /**
     * This method lets the user to add a new rock to the diary (displays
     * the appropriate dialog).
     */
    private void addRock ()
    {
        JCDTreeNode selNode = getSelectedNode ();

        DiaryItem diTemp = new DiaryItem (DiaryItem.TYPE_ROCK, "", "", false, -1, true, "");
        GroupDialog dialog = new GroupDialog (diTemp);
        dialog.setVisible (true);

        if (dialog.isDialogCanceled () == false) {
            ((JCDDocument) JCDApp.getDocument ()).insertRock (diTemp, selNode);
        }
    }

    /**
     * This method lets the user to add a new route to the diary (displays
     * the appropriate dialog).
     */
    private void addRoute ()
    {
        JCDTreeNode selNode = getSelectedNode ();

        // create an empty diary entry for the dialog
        DiaryItem beResult = new DiaryItem (DiaryItem.TYPE_ROUTE, "", "", false, -1, true, "");
        RouteDialog dialog = new RouteDialog (beResult);
        dialog.setVisible (true);

        // add route to diary tree only if filled correctly
        if (dialog.isFilledCorrect () == true) {
            beResult = dialog.getDiaryItem ();
            ((JCDDocument) JCDApp.getDocument ()).insertRoute (beResult, selNode);
        }
    }

    /**
     * This method lets the user to edit an existing diary item (displays
     * the appropriate dialog).
     */
    public void editItem ()
    {
        JCDTreeNode selNode = getSelectedNode ();

        if ((selNode == null) || (selNode == ((JCDDocument) JCDApp.getDocument ()).rootNode))
        {
            JOptionPane.showMessageDialog (JCDApp.getFrame (),
                JCDApp.getResReader ().getResourceString ("mbox.no_item_selected.text"),
                JCDApp.getResReader ().getResourceString ("mbox.no_item_selected.title"),
                JOptionPane.ERROR_MESSAGE);
            return;
        }

        DiaryItem diTemp = selNode.getDiaryItem ();
        
        // edit properties of area, group or rock
        if ((diTemp.type >= 0) && (diTemp.type <= 2))
        {
            GroupDialog dialog = new GroupDialog (diTemp);
            dialog.setVisible (true);

            // only apply the changes, when OK was pressed in dialog
            if (dialog.isDialogCanceled () == false)
                ((JCDDocument) JCDApp.getDocument ()).fAnyChanges = true;
        }
        // edit properties of route
        else {
            RouteDialog dialog = new RouteDialog (diTemp);
            dialog.setVisible (true);
            selNode.setDiaryItem (dialog.getDiaryItem ());

            // only apply the changes, when OK was pressed in dialog
            if (dialog.isFilledCorrect () == true)
                ((JCDDocument) JCDApp.getDocument ()).fAnyChanges = true;
        }
    }

    /**
     * Count routes beneath a diary entry. 
     */
    private void countRoutes ()
    {
        JCDTreeNode selNode = getSelectedNode ();

        if (selNode == null) {
            return;
        }
        
        DiaryItem diTemp = diTemp = selNode.getDiaryItem ();        
        int[] entriesCount = ((JCDDocument) JCDApp.getDocument ()).countEntries (selNode);

        CountEntriesDialog dialog = new CountEntriesDialog (diTemp.name, entriesCount[0], entriesCount[1], entriesCount[2], entriesCount[3], entriesCount[4]);
        dialog.setVisible (true);
    }

    /**
     * This method deletes the selected diary entry.
     */
    private void deleteItem ()
    {
        JCDTreeNode selNode = getSelectedNode () ;
        ((JCDDocument) JCDApp.getDocument ()).deleteTreeNode (selNode);
    }

    /**
     * This method displays the popup menu for the current diary item.
     */
    public void showPopupMenu (int xpos, int ypos)
    {
        // update menu item status first
        setMenuItemsStatus ();
        pmEntry.show (tree, xpos, ypos);
    }

    /**
     * This method enables or diables the menu items of the application
     * depending on the currently selected diary item (e.g. "Add rock" is
     * disabled, when a route is selected).
     */
    private void setMenuItemsStatus ()
    {
        JCDTreeNode selNode;

        if ((selNode = getSelectedNode ()) == null) {
            // nothing is selected
            miAddGroup.setEnabled (false);
            miAddRock.setEnabled (false);
            miAddRoute.setEnabled (false);
            pmiAddGroup.setEnabled (false);
            pmiAddRock.setEnabled (false);
            pmiAddRoute.setEnabled (false);
            miProperties.setEnabled (false);
            tbProperties.setEnabled (false);
            pmiProperties.setEnabled (false);
            miCountRoutes.setEnabled (false);
            pmiCountRoutes.setEnabled (false);
            tbCountRoutes.setEnabled (false);
            miDelete.setEnabled (false);
            pmiDelete.setEnabled (false);
            tbDelete.setEnabled (false);
            return;
        }

        DiaryItem diTemp = selNode.getDiaryItem ();

        if (selNode == ((JCDDocument) JCDApp.getDocument ()).rootNode) {
            // the root is selected
            miAddGroup.setEnabled (false);
            miAddRock.setEnabled (false);
            miAddRoute.setEnabled (false);
            pmiAddGroup.setEnabled (false);
            pmiAddRock.setEnabled (false);
            pmiAddRoute.setEnabled (false);
            miProperties.setEnabled (false);
            tbProperties.setEnabled (false);
            pmiProperties.setEnabled (false);
            miCountRoutes.setEnabled (true);
            pmiCountRoutes.setEnabled (true);
            tbCountRoutes.setEnabled (true);
            miDelete.setEnabled (false);
            pmiDelete.setEnabled (false);
            tbDelete.setEnabled (false);
        }

        if ((diTemp.type == DiaryItem.TYPE_AREA) || (diTemp.type == DiaryItem.TYPE_GROUP)) {
            // an area or a group is selected
            miAddGroup.setEnabled (true);
            miAddRock.setEnabled (true);
            miAddRoute.setEnabled (false);
            pmiAddGroup.setEnabled (true);
            pmiAddRock.setEnabled (true);
            pmiAddRoute.setEnabled (false);
            miProperties.setEnabled (true);
            pmiProperties.setEnabled (true);
            tbProperties.setEnabled (true);
            miCountRoutes.setEnabled (true);
            pmiCountRoutes.setEnabled (true);
            tbCountRoutes.setEnabled (true);
            miDelete.setEnabled (true);
            pmiDelete.setEnabled (true);
            tbDelete.setEnabled (true);
        }

        if (diTemp.type == DiaryItem.TYPE_ROCK) {
            // a rock is selected
            miAddGroup.setEnabled (false);
            miAddRock.setEnabled (false);
            miAddRoute.setEnabled (true);
            pmiAddGroup.setEnabled (false);
            pmiAddRock.setEnabled (false);
            pmiAddRoute.setEnabled (true);
            miProperties.setEnabled (true);
            pmiProperties.setEnabled (true);
            tbProperties.setEnabled (true);
            miCountRoutes.setEnabled (true);
            pmiCountRoutes.setEnabled (true);
            tbCountRoutes.setEnabled (true);
            miDelete.setEnabled (true);
            pmiDelete.setEnabled (true);
            tbDelete.setEnabled (true);
        }

        if (diTemp.type == DiaryItem.TYPE_ROUTE) {
            // a route is selected
            miAddGroup.setEnabled (false);
            miAddRock.setEnabled (false);
            miAddRoute.setEnabled (false);
            pmiAddGroup.setEnabled (false);
            pmiAddRock.setEnabled (false);
            pmiAddRoute.setEnabled (false);
            miProperties.setEnabled (true);
            pmiProperties.setEnabled (true);
            tbProperties.setEnabled (true);
            miCountRoutes.setEnabled (false);
            pmiCountRoutes.setEnabled (false);
            tbCountRoutes.setEnabled (false);
            miDelete.setEnabled (true);
            pmiDelete.setEnabled (true);
            tbDelete.setEnabled (true);
        }
    }

    /**
     * This method asks the user, if he really wants to append the srcNode
     * diary item to the destNode diary item (drag and drop operation).
     * @param srcNode the source of the D&D operation
     * @param destNode the destination of the D&D operation
     */
    public void moveDiaryItem (JCDTreeNode srcNode, JCDTreeNode destNode)
    {
        DiaryItem beSource = srcNode.getDiaryItem ();
        DiaryItem beDest = destNode.getDiaryItem ();

        // create message text
        int insertPos;
        String message = new String (JCDApp.getResReader ().getResourceString ("mbox.move_entry.text"));

        // enter the source name into the first '' string
        if ((insertPos = message.indexOf ("''")) != -1) {
            StringBuffer strTemp = new StringBuffer (message);
            strTemp = strTemp.insert (insertPos + 1, beSource.name);
            message = strTemp.toString ();
        }

        // enter the destination name into the second '' string
        if ((insertPos = message.indexOf ("''")) != -1) {
            StringBuffer strTemp = new StringBuffer (message);
            strTemp = strTemp.insert (insertPos + 1, beDest.name);
            message = strTemp.toString ();
        }

        if (JOptionPane.showConfirmDialog (JCDApp.getFrame (), message,
                JCDApp.getResReader ().getResourceString ("mbox.move_entry.title"),
                JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE)
            == JOptionPane.YES_OPTION)
        {
            ((JCDDocument) JCDApp.getDocument ()).moveDiaryItem (srcNode, destNode);
        }
    }

    /**
     * Quits application (save data first, if the user wants it).
     */
    private void quitApplication ()
    {
        if (((JCDDocument) JCDApp.getDocument ()).writeXMLFile () == false)
            return;
        
        // store the geometry properties of the frame
        JCDApp.getPreferences ().putInt ("mainwindow.x", JCDApp.getFrame ().getX ());
        JCDApp.getPreferences ().putInt ("mainwindow.y", JCDApp.getFrame ().getY ());
        JCDApp.getPreferences ().putInt ("mainwindow.width", JCDApp.getFrame ().getWidth ());
        JCDApp.getPreferences ().putInt ("mainwindow.height", JCDApp.getFrame ().getHeight ());
        System.exit (0);
    }

    /**
     * This method shows the Info-MessageBox.
     */
    private void showInfoBox ()
    {
        // abouttext consists of appname, version und abouttext
        String mbText = JCDApp.getResReader ().getResourceString ("app.name") + " " +
                        JCDApp.getResReader ().getResourceString ("app.version") + "\n" +
                        JCDApp.getResReader ().getResourceString ("mbox.about_box.text");

        URL urlLogo = this.getClass ().getResource ("/resource/images/jcdiary.gif");
        Icon jgbIcon = (Icon) new ImageIcon (urlLogo);
        JOptionPane.showMessageDialog (JCDApp.getFrame (), mbText,
            JCDApp.getResReader ().getResourceString ("mbox.about_box.title"),
            JOptionPane.INFORMATION_MESSAGE, jgbIcon);
    }


    /** This are listener classes for actions, that can be started in different ways. */
    
    /** 
     * Listener for exporting the diary. 
     */
    class ExportDiaryListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            ((JCDDocument) JCDApp.getDocument ()).exportDiaryData ();
        }
    }

    /** 
     * Listener for editing an entry. 
     */
    class EntryEditListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            editItem ();
        }
    }

    /** 
     * Listener for deleting an entry. 
     */
    class EntryDeleteListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            deleteItem ();
        }
    }

    /** 
     * Listener for count routes. 
     */
    class CountRoutesListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            countRoutes ();
        }
    }

    /** 
     * Listener for search entries. 
     */
    class EntryFindListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            SearchDialog sd = new SearchDialog ();
            sd.show ();
        }
    }

    /**
     * Listener for show online help window. 
     */
    class OnlineHelpListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            HelpViewer hv = new HelpViewer ();
        }
    }

    /**
     * Listener for show about information. 
     */
    class AppAboutListener implements ActionListener {
        public void actionPerformed (ActionEvent e) {
            showInfoBox ();
        }
    }
}

