package de.saring.jcdiary.dialogs;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import de.saring.jcdiary.JCDApp;

/**
 * This dialog is for showing the number of entries under the current selected
 * diary item. Example: if an group is currently selected, then the numbers of 
 * sub-groups, sub-rocks, sub-completed-routes and sub-uncompleted-routes will
 * be displayed (when available).
 *
 * @author  Stefan Saring
 * @version 1.0
 */
public class CountEntriesDialog extends JDialog
{
    
    /** 
     * Creates a new instance of CountEntriesDialog, which displays the number of entries
     * under the selected entry. When one oif the following parameters is 0, then the 
     * entry type will not be shown in the dialog.
     * @param entryName name of the currently selected entry
     * @param areas number of areas under the current entry
     * @param groups number of groups under the current entry
     * @param rocks number of rocks under the current entry
     * @param routesCompleted number of completed routes under the current entry
     * @param routesUncompleted number of uncompleted routes under the current entry
     */
    public CountEntriesDialog (String entryName, int areas, int groups, int rocks, int routesCompleted, int routesUncompleted)
    {
        super (JCDApp.getFrame (), true);
        initGUI (entryName, areas, groups, rocks, routesCompleted, routesUncompleted);

        this.setTitle (JCDApp.getResReader ().getResourceString ("count.title"));
        setResizable (false);
        setLocationRelativeTo (JCDApp.getFrame ());        
        
        // add WindowListener (close by default)
        addWindowListener (new WindowAdapter () {
            public void windowClosing (WindowEvent e) {
                dispose ();
            }
        });
    }


    /**
     * Constructs the GUI of this dialog. For parameters infos see the JavaDoc
     * of the constructor.
     */
    private void initGUI (String entryName, int areas, int groups, int rocks, int routesCompleted, int routesUncompleted)
    {
        JPanel pRoot = new JPanel (new BorderLayout ());
        JPanel pCounts = new JPanel (new GridBagLayout ());
        JPanel pBottom = new JPanel (new FlowLayout ());
        
        // create and add intro label
        JLabel lIntro = new JLabel (JCDApp.getResReader ().getResourceString ("count.intro") + " " + entryName + ":");
        pCounts.add (lIntro, new GridBagConstraints (0, 0, 2, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (10, 10, 10, 15), 0, 0));

        // do we need to show the number of areas ?
        if (areas > 0)
        {
            // create label for entry type and cout and show them
            JLabel lEntry = new JLabel (JCDApp.getResReader ().getResourceString ("count.areas"));
            JLabel lEntryCount = new JLabel (String.valueOf (areas));
            pCounts.add (lEntry, new GridBagConstraints (0, 1, 1, 1, 0f, 0f, 
                GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets (5, 30, 0, 5), 0, 0));
            pCounts.add (lEntryCount, new GridBagConstraints (1, 1, 1, 1, 0f, 0f, 
                GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (5, 5, 0, 20), 0, 0));
        }
        
        // do we need to show the number of groups ?
        if (groups > 0)
        {
            // create label for entry type and cout and show them
            JLabel lEntry = new JLabel (JCDApp.getResReader ().getResourceString ("count.groups"));
            JLabel lEntryCount = new JLabel (String.valueOf (groups));
            pCounts.add (lEntry, new GridBagConstraints (0, 2, 1, 1, 0f, 0f, 
                GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets (5, 30, 0, 5), 0, 0));
            pCounts.add (lEntryCount, new GridBagConstraints (1, 2, 1, 1, 0f, 0f, 
                GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (5, 5, 0, 20), 0, 0));
        }
        
        // do we need to show the number of rocks ?
        if (rocks > 0)
        {
            // create label for entry type and cout and show them
            JLabel lEntry = new JLabel (JCDApp.getResReader ().getResourceString ("count.rocks"));
            JLabel lEntryCount = new JLabel (String.valueOf (rocks));
            pCounts.add (lEntry, new GridBagConstraints (0, 3, 1, 1, 0f, 0f, 
                GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets (5, 30, 0, 5), 0, 0));
            pCounts.add (lEntryCount, new GridBagConstraints (1, 3, 1, 1, 0f, 0f, 
                GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (5, 5, 0, 20), 0, 0));
        }
        
        // do we need to show the number of completed completed routes ?
        if (routesCompleted > 0)
        {
            // create label for entry type and cout and show them
            JLabel lEntry = new JLabel (JCDApp.getResReader ().getResourceString ("count.routes"));
            JLabel lEntryCount = new JLabel (String.valueOf (routesCompleted) + "  " + JCDApp.getResReader ().getResourceString ("count.completed"));
            pCounts.add (lEntry, new GridBagConstraints (0, 4, 1, 1, 0f, 0f, 
                GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets (5, 30, 0, 5), 0, 0));
            pCounts.add (lEntryCount, new GridBagConstraints (1, 4, 1, 1, 0f, 0f, 
                GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (5, 5, 0, 20), 0, 0));
        }
        
        // do we need to show the number of completed uncompleted routes ?
        if (routesUncompleted > 0)
        {
            // if we have shown completed routes we don't need the entry label one more time
            if (routesCompleted <= 0) {
                JLabel lEntry = new JLabel (JCDApp.getResReader ().getResourceString ("count.routes"));
                pCounts.add (lEntry, new GridBagConstraints (0, 5, 1, 1, 0f, 0f, 
                    GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets (5, 30, 0, 5), 0, 0));
            }
            // create label for entry cout and show it
            JLabel lEntryCount = new JLabel (String.valueOf (routesUncompleted) + "  " + JCDApp.getResReader ().getResourceString ("count.uncompleted"));
            pCounts.add (lEntryCount, new GridBagConstraints (1, 5, 1, 1, 0f, 0f, 
                GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (5, 5, 0, 20), 0, 0));
        }
                
        // create panel for and "Close" button
        JButton bClose = new JButton (JCDApp.getResReader ().getResourceString ("count.close"));
        pBottom.add (bClose);        
        
        // add listener for close dialog button
        bClose.addActionListener (new ActionListener () {
            public void actionPerformed (ActionEvent e) {
                dispose ();
            }
        });        
        
        // add panels to dialog
        pBottom.setBorder (new EmptyBorder (15, 10, 10, 10));
        pRoot.add (pCounts, BorderLayout.CENTER);
        pRoot.add (pBottom, BorderLayout.SOUTH);
        getContentPane ().add (pRoot);
        pack ();
    }
}
