package de.saring.jcdiary.dialogs;

import de.saring.jcdiary.export.XSLDiaryExporter;
import java.awt.Cursor;
import java.io.*;
import java.net.URL;
import javax.swing.*;
import org.jdom.Document;
import de.saring.jcdiary.*;
import de.saring.jcdiary.tree.JCDTreeNode;
import de.saring.jcdiary.xml.JDOMTreeWriter;

/** 
 * This dialog is the frontend for the export diary feature. It was created using the
 * NetBeans Form editor, so it looks a bit strange :-).
 *
 * @author Stefan Saring
 * @version 2.0
 */
public class ExportDialog extends javax.swing.JDialog 
{
    /** The GUI controls. */
    private JPanel rootPanel;
    private JPanel firstPanel;
    private JLabel lFilename;
    private JTextField tfFilename;
    private JButton btBrowse;
    private JPanel secondPanel;
    private JLabel lStart;
    private JRadioButton rbRoot;
    private JRadioButton rbSelection;
    private JPanel thirdPanel;
    private JButton btExport;
    private JButton btCancel;

    /** 
     * Creates new form ExportDialog.
     */
    public ExportDialog ()
    {
        super (JCDApp.getFrame (), true);
        initComponents ();
        initDialog ();
        pack ();
        setLocationRelativeTo (JCDApp.getFrame ());        
    }

    /**
     * Initializes the dialog GUI.
     */
    public void initDialog ()
    {
        // additional GUI initializing
        ButtonGroup bg = new ButtonGroup ();
        bg.add (rbRoot);
        bg.add (rbSelection);
        rbRoot.setSelected (true);
        
        // disable 'Selected'-radiobutton, when no selection in diary tree
        if (((JCDView) JCDApp.getView ()).getSelectedNode () == null) {
            rbSelection.setEnabled (false);
        }
        
        // set localized text for controls
        this.setTitle (JCDApp.getResReader ().getResourceString ("export.dialog.title"));
        lFilename.setText (JCDApp.getResReader ().getResourceString ("export.file.label"));
        tfFilename.setText (JCDApp.getResReader ().getResourceString ("export.file.input"));
        btBrowse.setText (JCDApp.getResReader ().getResourceString ("export.file.browse"));
        lStart.setText (JCDApp.getResReader ().getResourceString ("export.from.label"));
        rbRoot.setText (JCDApp.getResReader ().getResourceString ("export.from.root"));
        rbSelection.setText (JCDApp.getResReader ().getResourceString ("export.from.selected"));
        btExport.setText (JCDApp.getResReader ().getResourceString ("export.export"));
        btCancel.setText (JCDApp.getResReader ().getResourceString ("export.cancel"));
        
        tfFilename.setText ("export.html");
    }
    
    /** 
     * This method is called from within the constructor to initialize the form.
     */
    private void initComponents ()
    {
        rootPanel = new javax.swing.JPanel ();
        firstPanel = new javax.swing.JPanel ();
        lFilename = new javax.swing.JLabel ();
        tfFilename = new javax.swing.JTextField ();
        btBrowse = new javax.swing.JButton ();
        secondPanel = new javax.swing.JPanel ();
        lStart = new javax.swing.JLabel ();
        rbRoot = new javax.swing.JRadioButton ();
        rbSelection = new javax.swing.JRadioButton ();
        thirdPanel = new javax.swing.JPanel ();
        btExport = new javax.swing.JButton ();
        btCancel = new javax.swing.JButton ();
        setDefaultCloseOperation (javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle ("Export diary data");
        addWindowListener (new java.awt.event.WindowAdapter ()
        {
            public void windowClosing (java.awt.event.WindowEvent evt)
            {
                closeDialog (evt);
            }
        }
        );

        rootPanel.setLayout (new java.awt.GridLayout (3, 1));
        rootPanel.setBorder (new javax.swing.border.EmptyBorder (new java.awt.Insets (10, 10, 10, 10)));

        firstPanel.setLayout (new java.awt.FlowLayout (0, 5, 5));  
        lFilename.setText ("lFilename");    
        firstPanel.add (lFilename);    
        tfFilename.setColumns (20);    
        firstPanel.add (tfFilename);
    
        btBrowse.setText ("btBrowse");
        btBrowse.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt)
            {
                btBrowseActionPerformed (evt);
            }
        });
    
        firstPanel.add (btBrowse);
        rootPanel.add (firstPanel);  
        secondPanel.setLayout (new java.awt.FlowLayout (0, 5, 5));  
        lStart.setText ("lStart");    
        secondPanel.add (lStart);    
        rbRoot.setText ("rbRoot");    
        secondPanel.add (rbRoot);    
        rbSelection.setText ("rbSelection");    
        secondPanel.add (rbSelection);    
        rootPanel.add (secondPanel);
    
        btExport.setText ("btExport");
        btExport.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt)
            {
                btExportActionPerformed (evt);
            }
        });
    
        thirdPanel.add (btExport);    
        btCancel.setText ("btCancel");
        btCancel.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt)
            {
                btCancelActionPerformed (evt);
            }
        });
    
        thirdPanel.add (btCancel);    
        rootPanel.add (thirdPanel);  

        getContentPane ().add (rootPanel, java.awt.BorderLayout.CENTER);
    }

    /** 
     * Export the diary data to the destination filename. 
     * @parame e the appropriate ActionEvent
     */
    private void btExportActionPerformed (java.awt.event.ActionEvent evt) 
    {
        // check target filename
        String strFilename = tfFilename.getText ();
        strFilename = strFilename.trim ();
        
        if (strFilename.equals ("")) {
            JOptionPane.showMessageDialog (this,
                JCDApp.getResReader ().getResourceString ("mbox.export.filename.text"),
                JCDApp.getResReader ().getResourceString ("mbox.export.filename.title"),
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        getContentPane ().setCursor (Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR));
        
        // setup the root node for export data
        JCDTreeNode rootNode4Export = null;
        if (rbRoot.isSelected () == true) {
            // export everything from root node
            rootNode4Export = ((JCDDocument) JCDApp.getDocument ()).rootNode;
        }
        else {
            // create a new root node, which contains only the selected nodes diary items
            rootNode4Export = ((JCDView) JCDApp.getView ()).getSelectedNode ().getRootNodeWithThisNodesContentOnly ();
        }

        try {
            // get stylesheet URL
            URL stylesheetURL = this.getClass ().getResource ("/resource/stylesheets/cldiary2html.xsl");
            
            // finally start the export 
            XSLDiaryExporter exporter = new XSLDiaryExporter (rootNode4Export);
            exporter.exportDiary (stylesheetURL, strFilename);
        }
        catch (Exception e) {
            System.err.println ("Exception: " + e.getMessage ());
            getContentPane ().setCursor (Cursor.getDefaultCursor ());
            JOptionPane.showMessageDialog (this,
                JCDApp.getResReader ().getResourceString ("mbox.export.export.text"),
                JCDApp.getResReader ().getResourceString ("mbox.export.export.title"),
                JOptionPane.ERROR_MESSAGE);
            return;
        }
        
        this.dispose ();
    }

    /** 
     * Opens the swing-filechooser for browsing the export filenme. 
     * @parame e the appropriate ActionEvent
     */
    private void btBrowseActionPerformed (java.awt.event.ActionEvent evt) 
    {
        javax.swing.JFileChooser fc = new javax.swing.JFileChooser (System.getProperty ("user.dir"));
        fc.setDialogTitle (JCDApp.getResReader ().getResourceString ("export.browsedialog.title"));
        fc.setSelectedFile (new java.io.File (tfFilename.getText ()));

        // has 'Save' been pressed ? -> replace textfield content witg browsed name
        if (fc.showSaveDialog (this) == javax.swing.JFileChooser.APPROVE_OPTION)
        {
            java.io.File file = fc.getSelectedFile ();
            tfFilename.setText (file.getAbsolutePath ());
        }
    }

    /** 
     * Cancels the dialog. 
     * @parame e the appropriate ActionEvent
     */
    private void btCancelActionPerformed (java.awt.event.ActionEvent evt) 
    {
        this.dispose ();
    }

    /** 
     * Closes the dialog 
     * @parame e the appropriate ActionEvent
     */
    private void closeDialog(java.awt.event.WindowEvent evt) 
    {
        this.dispose ();
    }    
}
