package de.saring.jcdiary.dialogs;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.EmptyBorder;
import de.saring.jcdiary.JCDApp;
import de.saring.jcdiary.tree.DiaryItem;


/**
 * This class implements an dialog for editing route entries in the climbers diary.
 * It is possible to edit the name, the date, the lead, the complete flag, the 
 * difficulty and an comment of an climbing route.
 *
 * @author Stefan Saring
 * @version 2.0
 */
public class RouteDialog extends JDialog implements ActionListener
{
    private DiaryItem diaryItem;
    private JButton bOK, bCancel;
    private JTextField tfName, tfDifficulty, tfDay, tfYear;
    private JTextArea taComment;
    private JComboBox cbMonth;
    private JCheckBox chbLead, chbCompleted;
    private boolean filledCorrect = false;

    /**
     * Creates the route dialog for the specified diary item.
     * @param dItem route diary item to display
     */
    public RouteDialog (DiaryItem dItem)
    {
        super (JCDApp.getFrame (), true);
        this.diaryItem = dItem;
        setTitle (JCDApp.getResReader ().getResourceString ("route.title"));
        init_GUI ();
        setResizable (false);
        setLocationRelativeTo (JCDApp.getFrame ());        
        setDiaryItemData ();

        // add WindowListener (close by default)
        addWindowListener (new WindowAdapter () {
            public void windowClosing (WindowEvent e) {
                dispose ();
            }
        });
    }

    /**
     * Constructs GUI of dialog.
     */
    private void init_GUI ()
    {
        JPanel rootPane = new JPanel ();
        rootPane.setLayout (new GridBagLayout ());                
        rootPane.setBorder (new EmptyBorder (10, 10, 10, 10));        

        // create label and textfield for route name
        JLabel lName = new JLabel (JCDApp.getResReader ().getResourceString ("route.name"), JLabel.LEFT);
        tfName = new JTextField ("", 30);
        rootPane.add (lName, new GridBagConstraints (0, 0, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 0, 0, 5), 0, 0));
        rootPane.add (tfName, new GridBagConstraints (1, 0, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets (0, 5, 0, 0), 0, 0));

        // create date input fields
        JLabel lDate = new JLabel (JCDApp.getResReader ().getResourceString ("route.date"), JLabel.LEFT);
        tfDay = new JTextField ("", 3);
        cbMonth = new JComboBox ();
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.january"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.february"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.march"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.april"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.may"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.june"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.july"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.august"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.september"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.october"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.november"));
        cbMonth.addItem (JCDApp.getResReader ().getResourceString ("route.month.december"));
        tfYear = new JTextField ("", 6);

        FlowLayout flDateControls = new FlowLayout (FlowLayout.LEFT);
        flDateControls.setHgap (0);
        JPanel pDateControls = new JPanel (flDateControls);
        pDateControls.add (tfDay);
        pDateControls.add (new JLabel ("  "));
        pDateControls.add (cbMonth);
        pDateControls.add (new JLabel ("  "));
        pDateControls.add (tfYear);
        
        rootPane.add (lDate, new GridBagConstraints (0, 1, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 0, 0, 5), 0, 0));
        rootPane.add (pDateControls, new GridBagConstraints (1, 1, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 5, 0, 0), 0, 0));

        // create input filed for difficulty
        JLabel lDifficulty = new JLabel (JCDApp.getResReader ().getResourceString ("route.difficulty"), JLabel.LEFT);
        tfDifficulty = new JTextField ("", 10);
        
        rootPane.add (lDifficulty, new GridBagConstraints (0, 2, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 0, 0, 5), 0, 0));
        rootPane.add (tfDifficulty, new GridBagConstraints (1, 2, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 5, 0, 0), 0, 0));

        // create checkboxes for "lead" and "completed"
        JLabel lMisc = new JLabel (JCDApp.getResReader ().getResourceString ("route.other"), JLabel.LEFT);
        chbLead = new JCheckBox (JCDApp.getResReader ().getResourceString ("route.lead"));
        chbCompleted = new JCheckBox (JCDApp.getResReader ().getResourceString ("route.completed"));

        FlowLayout flFlags = new FlowLayout (FlowLayout.LEFT);
        flFlags.setHgap (0);        
        JPanel pFlags = new JPanel (flFlags);
        pFlags.add (chbLead);
        pFlags.add (new JLabel ("      "));
        pFlags.add (chbCompleted);

        rootPane.add (lMisc, new GridBagConstraints (0, 3, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 0, 0, 5), 0, 0));
        rootPane.add (pFlags, new GridBagConstraints (1, 3, 1, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (0, 5, 0, 0), 0, 0));               

        // create label for group comment
        JLabel lComment = new JLabel (JCDApp.getResReader ().getResourceString ("route.comment"), JLabel.LEFT);
        rootPane.add (lComment, new GridBagConstraints (0, 4, 2, 1, 0f, 0f, 
            GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets (15, 0, 5, 0), 0, 0));
        
        // create textarea for group comment
        taComment = new JTextArea ();
        taComment.setColumns (40);
        taComment.setRows (6);
        taComment.setLineWrap (true);
        taComment.setWrapStyleWord (true);
        JScrollPane spComment = new JScrollPane ();
        spComment.getViewport ().add (taComment);
        rootPane.add (spComment, new GridBagConstraints (0, 5, 2, 1, 0f, 0f, 
            GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets (0, 0, 0, 0), 0, 0));
        
        // create OK and Cancel buttons
        bOK = new JButton (JCDApp.getResReader ().getResourceString ("group.ok"));
        bOK.addActionListener (this);
        bCancel = new JButton (JCDApp.getResReader ().getResourceString ("group.cancel"));
        bCancel.addActionListener (this);
        
        FlowLayout flButtons = new FlowLayout ();
        flButtons.setHgap (10);
        JPanel pButtons = new JPanel (flButtons);
        pButtons.add (bOK);
        pButtons.add (bCancel);
        rootPane.add (pButtons, new GridBagConstraints (0, 6, 2, 1, 0f, 0f, 
            GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets (10, 0, 0, 0), 0, 0));

        // add root pane 
        getContentPane ().add (rootPane);
        pack ();
    }

    /**
     * Fill GUI controls with diary item data.
     */
    private void setDiaryItemData ()
    {
        tfName.setText (diaryItem.name);
        tfDifficulty.setText (diaryItem.difficulty);
        chbLead.setSelected (diaryItem.lead);
        chbCompleted.setSelected (diaryItem.completed);
        taComment.setText (diaryItem.comment);

        // show current date, if date = -1
        if (diaryItem.date == -1)
        {
            Calendar cal = Calendar.getInstance ();
            int tTag = cal.get (cal.DATE);
            int tMonat = cal.get (cal.MONTH);
            int tJahr = cal.get (cal.YEAR);

            tfDay.setText (String.valueOf (tTag));
            cbMonth.setSelectedIndex (tMonat);
            tfYear.setText (String.valueOf (tJahr));
        }
        else {
            // decode date from long value
            // date = (day*1000000) + (month*10000) + year
            long helper;
            helper = diaryItem.date / 1000000;
            tfDay.setText (String.valueOf (helper));
            helper = (diaryItem.date % 1000000) / 10000;
            cbMonth.setSelectedIndex ((int) helper - 1);
            helper = (diaryItem.date % 10000);
            tfYear.setText (String.valueOf (helper));
        }
    }

    /**
     * Fill diary item object with GUI control values.
     */
    public void storeInputData ()
    {
        String tString;

        tString = tfName.getText ();
        if (tString.length () != 0)
            diaryItem.name = tString;
        else {
            displayError (JCDApp.getResReader ().getResourceString ("route.error.noname"));
            return;
        }

        tString = tfDifficulty.getText ();
        if (tString.length () != 0)
            diaryItem.difficulty = tString;
        else {
            displayError (JCDApp.getResReader ().getResourceString ("route.error.nodifficulty"));
            return;
        }

        // calculate date
        int tTag, tMonat, tJahr;
        try {
            Integer tInteger = Integer.valueOf (tfDay.getText ());
            tTag = tInteger.intValue ();
        }
        catch (NumberFormatException e) {
            displayError (JCDApp.getResReader ().getResourceString ("route.error.wrongday"));
            return;
        }

        if ((tTag < 1) || (tTag > 31)) {
            displayError (JCDApp.getResReader ().getResourceString ("route.error.wrongday"));
            return;
        }
        try {
            Integer tInteger = Integer.valueOf (tfYear.getText ());
            tJahr = tInteger.intValue ();
        }
        catch (NumberFormatException e) {
            displayError (JCDApp.getResReader ().getResourceString ("route.error.wrongyear"));
            return;
        }
        tMonat = cbMonth.getSelectedIndex () + 1;
        
        diaryItem.date = (tTag * 1000000) + (tMonat * 10000) + tJahr;
        diaryItem.lead = chbLead.isSelected ();
        diaryItem.completed = chbCompleted.isSelected ();
        diaryItem.comment = taComment.getText ();

        // set flag for correct route data
        filledCorrect = true;
        dispose ();
    }

    /**
     * Displays the appropriate error message.
     * @param sError message to display
     */
    public void displayError (String sError)
    {
        JOptionPane.showMessageDialog (JCDApp.getFrame (), sError,
            JCDApp.getResReader ().getResourceString ("route.error.title"),
            JOptionPane.ERROR_MESSAGE);
    }

    /**
     * Handle action events.
     * @param e the actionevent to handle
     */
    public void actionPerformed (ActionEvent e)
    {
        // Quelle des Events ermitteln
        Object source = e.getSource();

        if (source == bOK) {
            storeInputData ();
        }

        if (source == bCancel) {
            dispose ();
        }
    }

    /**
     * Returns the modified diary item.
     * @return the modifed item
     */
    public DiaryItem getDiaryItem ()
    {
        return diaryItem;
    }
    
    /**
     * Returns true, when the dialog was filled correctly by the user.
     * @return true when dialog filled correctly
     */
    public boolean isFilledCorrect ()
    {
        return filledCorrect;
    }
}
