package de.saring.jcdiary.dialogs;

import java.util.Enumeration;
import javax.swing.*;
import javax.swing.JOptionPane;
import javax.swing.tree.TreePath;
import de.saring.jcdiary.*;
import de.saring.jcdiary.tree.*;

/** 
 * This dialog is for searching entries, which contains a specific text in their
 * name or comment. It was created using the NetBeans Form editor, so it looks a 
 * bit strange :-).
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class SearchDialog extends javax.swing.JDialog 
{
    /** The GUI controls. */
    private JPanel rootPanel;
    private JPanel textPanel;
    private JLabel lSearchFor;
    private JTextField tfSearchFor;
    private JPanel commentPanel;
    private JCheckBox cbComments;
    private JPanel selectionPanel;
    private JCheckBox cbSelection;
    private JPanel buttonPanel;
    private JButton btSearch;
    private JButton btCancel;

    /** 
     * Creates new form SearchDialog.
     */
    public SearchDialog () 
    {
        super (JCDApp.getFrame (), true);
        initComponents ();
        initDialog ();
        pack ();
        setLocationRelativeTo (JCDApp.getFrame ());        
        
        // set focus to search text field
        tfSearchFor.selectAll ();
        tfSearchFor.requestFocus ();
    }

    /** 
     * Additional GUI initialization stuff. 
     */
    public void initDialog ()
    {
        // set localized text for controls
        this.setTitle (JCDApp.getResReader ().getResourceString ("search.title"));
        lSearchFor.setText (JCDApp.getResReader ().getResourceString ("search.text"));
        cbSelection.setText (JCDApp.getResReader ().getResourceString ("search.selection"));
        cbComments.setText (JCDApp.getResReader ().getResourceString ("search.comment"));
        btSearch.setText (JCDApp.getResReader ().getResourceString ("search.start"));
        btCancel.setText (JCDApp.getResReader ().getResourceString ("search.cancel"));
        
        // set the search settings from the last session
        tfSearchFor.setText (((JCDDocument) JCDApp.getDocument ()).lastSearchText);
        
        if (((JCDDocument) JCDApp.getDocument ()).fSearchInComments == true)
            cbComments.setSelected (true);
        else
            cbComments.setSelected (false);

        // selection-checkbox is only enabled, when a node is selected
        if (((JCDView) JCDApp.getView ()).getSelectedNode () == null) {
            cbSelection.setEnabled (false);
        }
        else {
            if (((JCDDocument) JCDApp.getDocument ()).fStartFromSelection == true)
                cbSelection.setSelected (true);
            else
                cbSelection.setSelected (false);
        }
        
        // add listener to textfield -> start search by pressing enter
        tfSearchFor.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt) {
                startSearch ();
            }
        });
    }
    
    /** 
     * This method is called from within the constructor to initialize the form.
     */
    private void initComponents ()
    {
        rootPanel = new javax.swing.JPanel ();
        textPanel = new javax.swing.JPanel ();
        lSearchFor = new javax.swing.JLabel ();
        tfSearchFor = new javax.swing.JTextField ();
        commentPanel = new javax.swing.JPanel ();
        cbComments = new javax.swing.JCheckBox ();
        selectionPanel = new javax.swing.JPanel ();
        cbSelection = new javax.swing.JCheckBox ();
        buttonPanel = new javax.swing.JPanel ();
        btSearch = new javax.swing.JButton ();
        btCancel = new javax.swing.JButton ();
        addWindowListener (new java.awt.event.WindowAdapter () {
            public void windowClosing (java.awt.event.WindowEvent evt)
            {
                closeDialog (evt);
            }
        });

        rootPanel.setLayout (new java.awt.GridLayout (4, 1));
        rootPanel.setBorder (new javax.swing.border.EmptyBorder (new java.awt.Insets (10, 10, 10, 10)));

        textPanel.setLayout (new java.awt.FlowLayout (0, 5, 5));  
        lSearchFor.setText ("lSearchFor");
        textPanel.add (lSearchFor);
        tfSearchFor.setColumns (20);
        textPanel.add (tfSearchFor);
        rootPanel.add (textPanel);
        commentPanel.setLayout (new java.awt.FlowLayout (0, 5, 5));
        cbComments.setText ("jCheckBox2");
        commentPanel.add (cbComments);
        rootPanel.add (commentPanel);
        selectionPanel.setLayout (new java.awt.FlowLayout (0, 5, 5));
        cbSelection.setText ("jCheckBox1");
        selectionPanel.add (cbSelection);
        rootPanel.add (selectionPanel);
  
        btSearch.setText ("jButton3");
        btSearch.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt)
            {
                btSearchActionPerformed (evt);
            }
        });
    
        buttonPanel.add (btSearch);    
        btCancel.setText ("jButton4");
        btCancel.addActionListener (new java.awt.event.ActionListener () {
            public void actionPerformed (java.awt.event.ActionEvent evt)
            {
                btCancelActionPerformed (evt);
            }
        });
    
        buttonPanel.add (btCancel);
        rootPanel.add (buttonPanel);
        getContentPane ().add (rootPanel, java.awt.BorderLayout.CENTER);
    }

    /** 
     * Search button was pressed. 
     * @param evt the appropriate ActionEvent.
     */
    private void btSearchActionPerformed (java.awt.event.ActionEvent evt) 
    {
        startSearch ();
    }

    /** 
     * Cancel button was pressed. 
     * @param evt the appropriate ActionEvent.
     */
    private void btCancelActionPerformed (java.awt.event.ActionEvent evt) 
    {
        closeSearchDialog ();
    }

    /** 
     * Closes the dialog 
     * @param evt the appropriate ActionEvent.
     */
    private void closeDialog(java.awt.event.WindowEvent evt) 
    {
        closeSearchDialog ();
    }

    /** 
     * Start a new search. 
     */
    public void startSearch ()
    {        
        // is the entered searchtext valid ?
        String searchText = tfSearchFor.getText ();
        if ((searchText == null) || (searchText.equals (""))) {
            JOptionPane.showMessageDialog (this,
                JCDApp.getResReader ().getResourceString ("mbox.search.notext.text"),
                JCDApp.getResReader ().getResourceString ("mbox.search.notext.title"),
                JOptionPane.ERROR_MESSAGE);

            // set focus to search text field
            tfSearchFor.selectAll ();
            tfSearchFor.requestFocus ();
            return;
        }

        // turn searchstring to lowercase and search for in all nodes
        if (searchNodes (searchText.toLowerCase ()) == false) 
        {
            // nothing was found
            JOptionPane.showMessageDialog (this,
                JCDApp.getResReader ().getResourceString ("mbox.search.notfound.text"),
                JCDApp.getResReader ().getResourceString ("mbox.search.notfound.title"),
                JOptionPane.ERROR_MESSAGE);
            
            // set focus to search text field
            tfSearchFor.selectAll ();
            tfSearchFor.requestFocus ();
            return;
        }
        // an entry was found -> exit
        closeSearchDialog ();
    }
    
    /** 
     * Searches all node for the specified text. 
     * @param searchText text to search for.
     * @return true, when something was found
     */
    public boolean searchNodes (String searchText)
    {
        // a found node is only valid, when selectedNode = null
        JCDTreeNode selectedNode = null;
        if (this.cbSelection.isSelected () == true)
            selectedNode = ((JCDView) JCDApp.getView ()).getSelectedNode ();
        
        // must the comments also be searched ?
        boolean searchComments = false;
        if (this.cbComments.isSelected () == true)
            searchComments = true;
        
        // get an list (depth first) of all treenodes (started with root)
        JCDTreeNode rootNode = ((JCDDocument) JCDApp.getDocument ()).rootNode;
        Enumeration nodeList = rootNode.preorderEnumeration ();

        // search all elements in the nodelist
        while (nodeList.hasMoreElements () == true)
        {
            // get name & comment (lowercase) of the current node
            JCDTreeNode node = (JCDTreeNode) nodeList.nextElement ();
            DiaryItem dItem = node.getDiaryItem ();
            String nodeName = dItem.name.toLowerCase ();
            String nodeComment = dItem.comment.toLowerCase ();
            
            // the current node is a valid match, when the searchtext is found in
            // the name or in the comment (if enabled) AND when the selectedNode
            // was passed
            if (((nodeName.indexOf (searchText) != -1) ||
                 ((searchComments == true) && (nodeComment.indexOf (searchText) != -1))) &&
                (selectedNode == null))
            {
                // select found node and ensure, that it is visible
                TreePath tpResult = new TreePath (node.getPath ());                 
                ((JCDView) JCDApp.getView ()).tree.setSelectionPath (tpResult);
                ((JCDView) JCDApp.getView ()).tree.scrollPathToVisible (tpResult);
                    
                return true;
            }

            // when current node is the selected -> next hit is valid
            if (node == selectedNode) {
                selectedNode = null;
            }
        }

        // no entry found
        return false;
    }
        
    /** 
     * Store search settings for next session and close dialog. 
     */
    private void closeSearchDialog ()
    {
        ((JCDDocument) JCDApp.getDocument ()).lastSearchText = tfSearchFor.getText ();
        ((JCDDocument) JCDApp.getDocument ()).fStartFromSelection = cbSelection.isSelected ();
        ((JCDDocument) JCDApp.getDocument ()).fSearchInComments = cbComments.isSelected ();
        setVisible (false);
        dispose ();
    }
}
