package de.saring.jcdiary.misc;

import java.awt.BorderLayout;
import java.awt.Image;
import java.awt.Cursor;
import java.awt.Container;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JEditorPane;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.HyperlinkEvent;
import javax.swing.text.Document;
import java.io.IOException;
import java.net.URL;
import de.saring.jcdiary.JCDApp;

/**
 * The class HelpViewer is an universal class for displaying help-pages (in html)
 * and can be used in all java applications. The help is displayed in an separate
 * (not modal) frame. The following of links is supported, but the new pages are
 * currently not loaded in an extra thread.
 *
 * @author Stefan Saring
 * @version 1.1
 */
public class HelpViewer extends JFrame implements HyperlinkListener
{
    private JEditorPane editorPane;
    private static boolean fHelpIsDisplayed;

    /**
     * Constructor of the HelpViewer. 
     */
    public HelpViewer ()
    {
        super ();
        
        // if there is allready an help-window displayed -> return
        if (fHelpIsDisplayed == true)
            return;
            
        // set the flag, that a helpwindow is currently shown
        fHelpIsDisplayed = true;
        
        // create GUI, title and frame-icon
        initGUI ();
        this.setTitle (JCDApp.getResReader ().getResourceString ("help.title"));
        URL urlLogo = this.getClass ().getResource ("/resource/images/jcdiary.gif");
        Image logo = getToolkit().getImage (urlLogo);
        this.setIconImage (logo);
        
        setStartPage ();
        this.setLocation (JCDApp.getPreferences ().getInt ("helpwindow.x", 20),
                          JCDApp.getPreferences ().getInt ("helpwindow.y", 20));
        this.setSize (JCDApp.getPreferences ().getInt ("helpwindow.width", 600),
                      JCDApp.getPreferences ().getInt ("helpwindow.height", 440));
                
        this.addWindowListener (new WindowAdapter () {
            public void windowClosing (WindowEvent e) {
                closeHelpWindow ();
            }
        });
        this.setVisible (true);
    }

    /**
     * Initializes the GUI of the helpwindow.
     */
    private void initGUI ()
    {
        editorPane = new JEditorPane ();
        editorPane.setEditable (false);
        editorPane.addHyperlinkListener (this);
        JScrollPane scrollPane = new JScrollPane (editorPane);
        this.getContentPane ().add (scrollPane, BorderLayout.CENTER);
        this.pack ();
    }
    
    /**
     * Closes the helpwindow. 
     */
    public void closeHelpWindow ()
    {
        // store the geometry properties of the helpwindow
        JCDApp.getPreferences ().putInt ("helpwindow.x", this.getX ());
        JCDApp.getPreferences ().putInt ("helpwindow.y", this.getY ());
        JCDApp.getPreferences ().putInt ("helpwindow.width", this.getWidth ());
        JCDApp.getPreferences ().putInt ("helpwindow.height", this.getHeight ());
        
        this.dispose ();
        fHelpIsDisplayed = false;
    }

    /** 
     * Sets the startpage of the help.
     */
    private void setStartPage ()
    {
        try {
            URL urlStartPage = this.getClass ().getResource (JCDApp.getResReader ().getResourceString ("help.startpage"));
            editorPane.setPage (urlStartPage.toString ());
        }
        catch (IOException e) {
            System.err.println ("Error: HelpViewer: " + e);
        }
    }

    
    /**
     * Implementation of the HyperlinkListener. 
     * @param e the appropriaty link event
     */
    public void hyperlinkUpdate (HyperlinkEvent e)
    {
        // only react, when link is pressed
    	if (e.getEventType () == HyperlinkEvent.EventType.ACTIVATED)
        {
            try {
                // show waitcursor while the page is beeing loaded
            	Cursor waitCursor = Cursor.getPredefinedCursor (Cursor.WAIT_CURSOR);
    	        editorPane.setCursor (waitCursor);
                // load the page to which the linl refers to
                editorPane.setPage (e.getURL ());
            }
            catch (IOException ioe) {
                getToolkit ().beep ();
            }
            // reset cursor
            Cursor defaultCursor = Cursor.getPredefinedCursor (Cursor.DEFAULT_CURSOR);
            editorPane.setCursor (defaultCursor);
    	}
    }
}
