package de.saring.jcdiary.tree;

import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.net.URL;

/**
 * Because every kind of entry uses its own icon a costumized class for rendering
 * the tree items is needed. This class enhances the standard TreeCellRenderer for
 * some features.
 *
 * @author Stefan Saring
 * @version 1.0
 */
public class JCDCellRenderer extends JLabel implements TreeCellRenderer
{
    static protected ImageIcon areaIcon;
    static protected ImageIcon groupIcon;
    static protected ImageIcon rockIcon;
    static protected ImageIcon routeIcon;
    static protected ImageIcon routeUncompleteIcon;
    static protected final Color SelectedBackgroundColor = Color.gray;
    static protected Color foregroundColor = null;


    /**
     * Constructor of renderer class.
     */
    public JCDCellRenderer ()
    {
        super ();

        // if the static icons are not loaded yet, do it now
        if ((areaIcon == null) || (groupIcon == null) ||
            (rockIcon == null) || (routeIcon == null))
        {
            URL urlIcon = this.getClass ().getResource ("/resource/images/area.gif");
            areaIcon = new ImageIcon (urlIcon);
            urlIcon = this.getClass ().getResource ("/resource/images/group.gif");
            groupIcon = new ImageIcon (urlIcon);
            urlIcon = this.getClass ().getResource ("/resource/images/rock.gif");
            rockIcon = new ImageIcon (urlIcon);
            urlIcon = this.getClass ().getResource ("/resource/images/route.gif");
            routeIcon = new ImageIcon (urlIcon);
            urlIcon = this.getClass ().getResource ("/resource/images/route-unc.gif");
            routeUncompleteIcon = new ImageIcon (urlIcon);
        }
    }

    // is the icon selected
    protected boolean selected;

    /**
     * This method creates the Label, which will be shown for the item.
     * @param any see API-doc for TreeCellRenderer
     * @return see API-doc for TreeCellRenderer
     */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
        boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus)
    {
        // remember the non-selected foreground color
        if (foregroundColor == null)
            foregroundColor = getForeground ();

        String stringValue = tree.convertValueToText(value, selected,
             expanded, leaf, row, hasFocus);

        // set text and tooltips
        setText(stringValue);
        setToolTipText(stringValue);

        // set icon
        DiaryItem diTemp = (DiaryItem) ((JCDTreeNode) value).getDiaryItem ();

        if (diTemp.type == 0) setIcon (areaIcon);
        if (diTemp.type == 1) setIcon (groupIcon);
        if (diTemp.type == 2) setIcon (rockIcon);
        if (diTemp.type == 3) 
        {
            // use separate icon for uncompleted routes
            if (diTemp.completed == true)
                setIcon (routeIcon);
            else
                setIcon (routeUncompleteIcon);
        }
        
        if (diTemp.type < 0) setIcon (null);

        if (hasFocus)
            setForeground (Color.white);
        else
            setForeground (foregroundColor);

        this.selected = selected;
        return this;
    }

    /**
     * Paint the label of the tree item.
     */
    public void paint (Graphics g)
    {
        Color bColor;
        Icon  currentI = getIcon ();

        if (selected)
            bColor = SelectedBackgroundColor;
        else
            if(getParent () != null)
                bColor = getParent ().getBackground ();
            else
                bColor = getBackground ();

        g.setColor (bColor);

        if (currentI != null && getText () != null)
        {
            int offset = (currentI.getIconWidth () + getIconTextGap ());
            g.fillRect(offset, 0, getWidth () - 1 - offset, getHeight () - 1);
        }
        else
            g.fillRect (0, 0, getWidth () - 1, getHeight () - 1);

        // let baseclass do the rest :)
        super.paint (g);
    }
}
